﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.Research.Kinect.Nui;

namespace SkeletalViewerCS
{
    public partial class SkeletalViewerCS : Form
    {
        Runtime nui;
        
        // We want to control how depth data gets converted into false-color data
        // for more intuitive visualization, so we keep 32-bit color frame buffer versions of
        // these, to be updated whenever we receive and process a 16-bit frame.
        const int RED_IDX = 2;
        const int GREEN_IDX = 1;
        const int BLUE_IDX = 0;
        byte[] depthFrame32 = new byte[320 * 240 * 4];

        frmVideo fVideo;
        frmDepth fDepth;
        frmSkeleton fSkeleton;

        public SkeletalViewerCS()
        {
            InitializeComponent();

            fVideo = new frmVideo();
            fVideo.Show();

            fDepth = new frmDepth();
            fDepth.Show();

            fSkeleton = new frmSkeleton();
            fSkeleton.Show();

        }

        private void SkeletalViewerCS_Load(object sender, EventArgs e)
        {
            nui = new Runtime();

            try
            {
                nui.Initialize(RuntimeOptions.UseDepthAndPlayerIndex | RuntimeOptions.UseSkeletalTracking | RuntimeOptions.UseColor);
            }
            catch (InvalidOperationException)
            {
                MessageBox.Show("Runtime initialization failed. Please make sure Kinect device is plugged in.");
                return;
            }

            try
            {
                nui.VideoStream.Open(ImageStreamType.Video, 2, ImageResolution.Resolution640x480, ImageType.Color);
                nui.DepthStream.Open(ImageStreamType.Depth, 2, ImageResolution.Resolution320x240, ImageType.DepthAndPlayerIndex);
            }
            catch (InvalidOperationException)
            {
                MessageBox.Show("Failed to open stream. Please make sure to specify a supported image type and resolution.");
                return;
            }

            nui.DepthFrameReady += new EventHandler<ImageFrameReadyEventArgs>(nui_DepthFrameReady);
            nui.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(nui_SkeletonFrameReady);
            nui.VideoFrameReady += new EventHandler<ImageFrameReadyEventArgs>(nui_ColorFrameReady);
        }

        private void SkeletalViewerCS_FormClosing(object sender, FormClosingEventArgs e)
        {
            nui.Uninitialize();
            Environment.Exit(0);
        }

        void nui_ColorFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            Bitmap bmp;

            // 32-bit per pixel, RGBA image
            PlanarImage Image = e.ImageFrame.Image;
            unsafe
            {
                fixed (byte* ptr = &Image.Bits[0])
                {
                    bmp = new Bitmap(Image.Width, Image.Height, Image.Width * 4,
                        System.Drawing.Imaging.PixelFormat.Format32bppRgb, (IntPtr)ptr);
                }
            }

            fVideo.grpVideo.DrawImage(bmp, 0, 0);
            fVideo.picVideo.Refresh();

        }



        // Converts a 16-bit grayscale depth frame which includes player indexes into a 32-bit frame
        // that displays different players in different colors
        byte[] convertDepthFrame(byte[] depthFrame16)
        {
            for (int i16 = 0, i32 = 0; i16 < depthFrame16.Length && i32 < depthFrame32.Length; i16 += 2, i32 += 4)
            {
                int player = depthFrame16[i16] & 0x07;
                int realDepth = (depthFrame16[i16 + 1] << 5) | (depthFrame16[i16] >> 3);
                // transform 13-bit depth information into an 8-bit intensity appropriate
                // for display (we disregard information in most significant bit)
                byte intensity = (byte)(255 - (255 * realDepth / 0x0fff));

                depthFrame32[i32 + RED_IDX] = 0;
                depthFrame32[i32 + GREEN_IDX] = 0;
                depthFrame32[i32 + BLUE_IDX] = 0;

                // choose different display colors based on player
                switch (player)
                {
                    case 0:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity / 2);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity / 2);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity / 2);
                        break;
                    case 1:
                        depthFrame32[i32 + RED_IDX] = intensity;
                        break;
                    case 2:
                        depthFrame32[i32 + GREEN_IDX] = intensity;
                        break;
                    case 3:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity / 4);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity);
                        break;
                    case 4:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity / 4);
                        break;
                    case 5:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity / 4);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity);
                        break;
                    case 6:
                        depthFrame32[i32 + RED_IDX] = (byte)(intensity / 2);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(intensity / 2);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(intensity);
                        break;
                    case 7:
                        depthFrame32[i32 + RED_IDX] = (byte)(255 - intensity);
                        depthFrame32[i32 + GREEN_IDX] = (byte)(255 - intensity);
                        depthFrame32[i32 + BLUE_IDX] = (byte)(255 - intensity);
                        break;
                }
            }
            return depthFrame32;
        }

        void nui_DepthFrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            Bitmap bmp;

            PlanarImage Image = e.ImageFrame.Image;
            byte[] convertedDepthFrame = convertDepthFrame(Image.Bits);

            unsafe
            {
                fixed (byte* ptr = &convertedDepthFrame[0])
                {
                    bmp = new Bitmap(Image.Width, Image.Height, Image.Width * 4,
                        System.Drawing.Imaging.PixelFormat.Format32bppRgb, (IntPtr)ptr);
                }
            }
            fDepth.grpDepth.DrawImage(bmp, 0, 0);
            fDepth.picDepth.Refresh();
        }


        private Point getDisplayPosition(Joint joint)
        {
            float depthX, depthY;
            nui.SkeletonEngine.SkeletonToDepthImage(joint.Position, out depthX, out depthY);
            depthX = Math.Max(0, Math.Min(depthX * 320, 320));  //convert to 320, 240 space
            depthY = Math.Max(0, Math.Min(depthY * 240, 240));  //convert to 320, 240 space
            int colorX, colorY;
            ImageViewArea iv = new ImageViewArea();
            // only ImageResolution.Resolution640x480 is supported at this point
            nui.NuiCamera.GetColorPixelCoordinatesFromDepthPixel(ImageResolution.Resolution640x480, iv, (int)depthX, (int)depthY, (short)0, out colorX, out colorY);

            // map back to skeleton.Width & skeleton.Height
            return new Point((int)(fSkeleton.picSkeleton.Width * colorX / 640.0), (int)(fSkeleton.picSkeleton.Height * colorY / 480));
        }

        void nui_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {

            Graphics g = fSkeleton.grpSkeleton;

            SkeletonFrame skeletonFrame = e.SkeletonFrame;
            int iSkeleton = 0;
            SolidBrush[] brushes = new SolidBrush[6];
            brushes[0] = new SolidBrush(Color.FromArgb(255, 0, 0));
            brushes[1] = new SolidBrush(Color.FromArgb(0, 255, 0));
            brushes[2] = new SolidBrush(Color.FromArgb(64, 255, 255));
            brushes[3] = new SolidBrush(Color.FromArgb(255, 255, 64));
            brushes[4] = new SolidBrush(Color.FromArgb(255, 64, 255));
            brushes[5] = new SolidBrush(Color.FromArgb(128, 128, 255));

            g.Clear(Color.Black);
            foreach (SkeletonData data in skeletonFrame.Skeletons)
            {
                if (SkeletonTrackingState.Tracked == data.TrackingState)
                {
                    //// Draw bones
                    Brush brush = brushes[iSkeleton % brushes.Length];
                    //skeleton.Children.Add(getBodySegment(data.Joints, brush, JointID.HipCenter, JointID.Spine, JointID.ShoulderCenter, JointID.Head));
                    //skeleton.Children.Add(getBodySegment(data.Joints, brush, JointID.ShoulderCenter, JointID.ShoulderLeft, JointID.ElbowLeft, JointID.WristLeft, JointID.HandLeft));
                    //skeleton.Children.Add(getBodySegment(data.Joints, brush, JointID.ShoulderCenter, JointID.ShoulderRight, JointID.ElbowRight, JointID.WristRight, JointID.HandRight));
                    //skeleton.Children.Add(getBodySegment(data.Joints, brush, JointID.HipCenter, JointID.HipLeft, JointID.KneeLeft, JointID.AnkleLeft, JointID.FootLeft));
                    //skeleton.Children.Add(getBodySegment(data.Joints, brush, JointID.HipCenter, JointID.HipRight, JointID.KneeRight, JointID.AnkleRight, JointID.FootRight));

                    // Draw joints
                    foreach (Joint joint in data.Joints)
                    {
                        Point jointPos = getDisplayPosition(joint);
                        //Line jointLine = new Line();
                        //jointLine.X1 = jointPos.X - 3;
                        //jointLine.X2 = jointLine.X1 + 6;
                        //jointLine.Y1 = jointLine.Y2 = jointPos.Y;
                        //jointLine.Stroke = jointColors[joint.ID];
                        //jointLine.StrokeThickness = 6;
                        //skeleton.Children.Add(jointLine);

                        g.FillEllipse(brush, jointPos.X - 3, jointPos.Y - 3, 6, 6);

                        
                    }
                }
                iSkeleton++;
            } // for each skeleton

            fSkeleton.picSkeleton.Invalidate();
        }

        private void btnUp_Click(object sender, EventArgs e)
        {
            try
            {
                nui.NuiCamera.ElevationAngle += 5;
            }
            catch { }
        }

        private void btnDown_Click(object sender, EventArgs e)
        {
            try
            {
                nui.NuiCamera.ElevationAngle -= 5;
            }
            catch { }
        }



    }
}
